import importlib
import inspect

from django.conf import settings
from django_q.tasks import async_task
from drf_spectacular.utils import extend_schema
from rest_framework import serializers, status
from rest_framework.response import Response
from rest_framework.views import APIView

from .helpers import BaseCronTask


class CronTaskRunSerializer(serializers.Serializer):
    params = serializers.DictField()


class CronTasksView(APIView):
    def get(self, request):
        cron_tasks = []
        for cron_task_class_path in settings.CRON_TASK_CLASSES:
            module_name, class_name = cron_task_class_path.rsplit('.', 1)
            module = importlib.import_module(module_name)
            cron_task_class: BaseCronTask = getattr(module, class_name)
            argspec = inspect.getfullargspec(cron_task_class.run)
            cron_tasks.append({
                'name': cron_task_class.name,
                'args': argspec.args,
            })

        return Response(cron_tasks)


class CronTaskRunView(APIView):
    @extend_schema(request=[CronTaskRunSerializer])
    def post(self, request, name):
        serializer = CronTaskRunSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        for cron_task_class_path in settings.CRON_TASK_CLASSES:
            module_name, class_name = cron_task_class_path.rsplit('.', 1)
            module = importlib.import_module(module_name)
            cron_task_class: BaseCronTask = getattr(module, class_name)
            if cron_task_class.name == name:
                run_func_name = cron_task_class_path + '.run'
                async_task(run_func_name, serializer.validated_data['params'])
                return Response()

        return Response(status=status.HTTP_404_NOT_FOUND)



from datetime import timedelta

from ..utils.date import EDate, EDateTime, to_edate


def resolve_date(date_or_days_ago: EDate | str | int, default=None):
    if isinstance(date_or_days_ago, EDate):
        return date_or_days_ago
    elif isinstance(date_or_days_ago, int):
        return to_edate(EDateTime.now().first_second_of_day() - timedelta(days=abs(date_or_days_ago)))
    elif isinstance(date_or_days_ago, str):
        return EDate.strptime(date_or_days_ago)
    else:
        return default or EDate.today().yesterday()


def resolve_month(date_or_months_ago: EDate | str | int, default=None):
    if isinstance(date_or_months_ago, EDate):
        return date_or_months_ago
    elif isinstance(date_or_months_ago, int):
        date = EDate.today().first_day_of_month()
        for _ in range(abs(date_or_months_ago)):
            date = date.first_day_of_last_month()
        return date
    elif isinstance(date_or_months_ago, str):
        return EDate.strptime(date_or_months_ago)
    else:
        return default or EDate.today().first_day_of_month()



from django.utils import translation
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from energy_base.translation import translate as _


class DashboardDateSerializer(serializers.Serializer):
    datetime = serializers.DateTimeField()
    period = serializers.ChoiceField(choices=['daily', 'hourly'], default='daily')


class DashboardSerializer(serializers.Serializer):
    datetime = serializers.DateTimeField(required=False)
    period = serializers.ChoiceField(choices=['hourly', 'today', 'daily', 'monthly', 'yearly'], default='daily')

    def validate(self, attrs):
        period = attrs.get('period', 'daily')
        datetime = attrs.get('datetime')

        if period != 'today' and datetime is None:
            raise ValidationError({
                'datetime': _('This field is required when period is not "today".')
            })
        return attrs


class TranslatedField(serializers.CharField):
    _AVAILABLE_LANGUAGES = ['uz', 'ru', 'crl']

    def get_language(self):
        if lang_from_header := self.context['request'].headers.get('accept-language'):
            if lang_from_header and lang_from_header not in self._AVAILABLE_LANGUAGES:
                lang_from_header = 'ru'

        return lang_from_header or translation.get_language()

    def get_attribute(self, instance):
        field_name = self.source or self.field_name

        lang = self.get_language()
        translated_field = f"{field_name}_{lang}"

        if hasattr(instance, translated_field):
            return getattr(instance, translated_field)

        return getattr(instance, field_name, None)



from django.db import models


class DeviationReasonType(models.TextChoices):
    NO_CONNECTION = 'no_connection', 'Нет связи с оборудованием'
    IN_RESERVE = 'in_reserve', 'В резерве'
    MALFUNCTION = 'malfunction', 'Неисправность устройства'
    OTHER = 'other', 'Другое'



from django.urls import path

from .views import CronTasksView, CronTaskRunView

urlpatterns = [
    path('cron-tasks', CronTasksView.as_view(), name='cron-tasks'),
    path('cron-tasks/<str:name>/run', CronTaskRunView.as_view(), name='cron-task-run'),
]
